# THIS POWERSHELL SCRIPT RETRIEVES THE FIVE CLOSEST WEATHER STATIONS TO A GIVEN LOCATION USING THE API.WEATHER.GOV API
# FOR EACH STATION, IT DISPLAYS:
# * STATION NAME AND ID
# * DISTANCE (MI/KM) FROM THE LOCATION
# * LOCAL OBSERVATION TIME
# * TEMPERATURE, HUMIDITY, WIND SPEED/DIRECTION
# * CONDITIONS SUMMARY (E.G. "PARTLY CLOUDY")
# * PRECIPITATION IN THE LAST 1 AND 3 HOURS
# * STATION ELEVATION (ft/m)
#
# IT ALSO 
# 1) DISPLAYS THE ELEVATION OF YOUR SPECIFIED LOCATION USING OPEN-ELEVATION, AND 
# 2) SAVES RESULTS TO BOTH .CSV AND .JSON IN A TARGET FOLDER
#
# Version: 2
# Designed by jaydeel at ipcamtalk.com
# Coded with assistance from ChatGPT
#
# Updates:
# V2 2025-04-22 bug fixes and refinements
# V1 2024-04-22 initial release

param (
    [double]$Latitude,
    [double]$Longitude,
    [switch]$UseImperial,
    [string]$OutPath = 'C:\temp'
)

# --- Hardcoded fallback location ---
if (-not $Latitude)  { $Latitude  = 37.7749 }   # Default: San Francisco
if (-not $Longitude) { $Longitude = -122.4194 }

# HELPER FUNCTIONS
if (1 -eq 1) {  # Use to collapse this code block while editing
 
    function AllBlank {
        param ([array]$Array)
        return ($Array -is [array]) -and ($Array | Where-Object { $_ -ne $null -and $_ -ne '' -and $_ -ne 'N/A' }).Count -eq 0
    }

    function AllSame {
        param ([array]$Array)
        if (-not ($Array -is [array]) -or $Array.Count -eq 0) { return $false }
        $first = $Array[0]
        return ($Array | Where-Object { $_ -ne $first }).Count -eq 0
    }

    function Get-DistanceKm {
        param ($lat1, $lon1, $lat2, $lon2)
        $R = 6371
        $dLat = [Math]::PI * ($lat2 - $lat1) / 180.0
        $dLon = [Math]::PI * ($lon2 - $lon1) / 180.0
        $a = [Math]::Pow([Math]::Sin($dLat / 2), 2) +
             [Math]::Cos([Math]::PI * $lat1 / 180.0) *
             [Math]::Cos([Math]::PI * $lat2 / 180.0) *
             [Math]::Pow([Math]::Sin($dLon / 2), 2)
        $c = 2 * [Math]::Atan2([Math]::Sqrt($a), [Math]::Sqrt(1 - $a))
        return $R * $c
    }

    function Convert-DegToCompass {
        param ([double]$degrees)
        $dirs = 'N','NNE','NE','ENE','E','ESE','SE','SSE','S','SSW','SW','WSW','W','WNW','NW','NNW'
        return $dirs[[math]::Round(($degrees % 360) / 22.5) % 16]
    }

    function Convert-CtoF     ($C)  { return ($C * 9 / 5) + 32 }
    function Convert-msToMph  ($ms) { return $ms * 2.23694 }
    function Convert-mmToIn   ($mm) { return $mm / 25.4 }
    function Convert-mToFt    ($m)  { return $m * 3.28084 }
    
}

if (-not (Test-Path -LiteralPath $OutPath)) {
    New-Item -ItemType Directory -Path $OutPath -Force | Out-Null
}

$point_url   = "https://api.weather.gov/points/$Latitude,$Longitude"
$point_data  = Invoke-RestMethod -Uri $point_url
$station_url = $point_data.properties.observationStations
$stations    = (Invoke-RestMethod -Uri $station_url).features

$elev_api_url = "https://api.open-elevation.com/api/v1/lookup?locations=$Latitude,$Longitude"
try {
    $elev_response = Invoke-RestMethod -Uri $elev_api_url -TimeoutSec 5
    $elev_m = $elev_response.results[0].elevation
} catch {
    $elev_m = 0
}
$elev_text = if ($UseImperial) {
    "{0} ft" -f ([math]::Round((Convert-mToFt $elev_m), 1))
} else {
    "{0} m" -f ([math]::Round($elev_m, 1))
}
Write-Host "`nElevation at specified location ($Latitude, $Longitude): $elev_text`n"

$distance_data = $stations | ForEach-Object {
    $lat2 = $_.geometry.coordinates[1]
    $lon2 = $_.geometry.coordinates[0]
    $dist_km = Get-DistanceKm -lat1 $Latitude -lon1 $Longitude -lat2 $lat2 -lon2 $lon2
    $dist_mi = $dist_km * 0.621371
    [PSCustomObject]@{
        ID           = $_.properties.stationIdentifier
        Name         = $_.properties.name
        Latitude     = $lat2
        Longitude    = $lon2
        Distance_km  = $dist_km
        Distance_mi  = $dist_mi
    }
} | Sort-Object (@{Expression={if ($UseImperial) {$_.Distance_mi} else {$_.Distance_km}}}) | Select-Object -First 5

$weather_data = $distance_data | ForEach-Object {

    $station_id = $_.ID
    $obs_url = "https://api.weather.gov/stations/$station_id/observations/latest"
    $obs = $null
    try {
        $obs = Invoke-RestMethod -Uri $obs_url -TimeoutSec 5
    } catch {
        return
    }

    $props = $obs.properties
    $tempC = $props.temperature.value
    $tempF = if ($tempC -ne $null) { [Math]::Round((Convert-CtoF $tempC), 1) }
    $humid = $props.relativeHumidity.value
    $windMs = $props.windSpeed.value
    $windMph = if ($windMs -ne $null) { [Math]::Round((Convert-msToMph $windMs), 1) }
    $windDirDeg = $props.windDirection.value
    $windDirStr = if ($windDirDeg -ne $null) { Convert-DegToCompass $windDirDeg } else { "N/A" }
    $weather = $props.textDescription

    $precip1 = $props.precipitationLastHour.value
    $precip3 = $props.precipitationLast3Hours.value

    $obs_time_local = if ($props.timestamp) {
        [datetime]::Parse($props.timestamp).ToLocalTime().ToString("yyyy-MM-dd HH:mm:ss")
    } else {
        "N/A"
    }

    $elev_station_m = $obs.properties.elevation.value
    $elev_val = if ($UseImperial) { [math]::Round((Convert-mToFt $elev_station_m), 1) } else { [math]::Round($elev_station_m, 1) }

    $dist_val = if ($UseImperial) { [math]::Round($_.Distance_mi, 2) } else { [math]::Round($_.Distance_km, 2) }
    $temp_val = if ($UseImperial) { $tempF } else { $tempC }
    $wind_val = if ($UseImperial) { $windMph } else { $windMs }
    $humid_val_floor = if ($humid -ne $null) { [math]::Floor($humid) } else { $null }

    [PSCustomObject]@{
        ID              = $_.ID
        Name            = $_.Name
        Name_Display    = ($_.Name -replace '\s+', ' ') -replace '^(.{25}).*$', '$1...'
        Distance        = $dist_val
        Elevation       = $elev_val
        Timestamp       = $obs_time_local
        Temp            = $temp_val
        Humidity        = $humid_val_floor
        WindSpeed       = $wind_val
        WindDir         = $windDirStr
        Conditions      = if ($weather) { $weather } else { "N/A" }
        Precip_1hr      = if ($precip1 -ne $null) { [math]::Round($precip1, 1) } else { "N/A" }
        Precip_3hr      = if ($precip3 -ne $null) { [math]::Round($precip3, 1) } else { "N/A" }
        HumidityRaw     = $humid
    }
}

$unit_dist = if ($UseImperial) { "mi" } else { "km" }
$unit_elev = if ($UseImperial) { "ft" } else { "m" }
$unit_temp = if ($UseImperial) { "F" } else { "C" }
$unit_wind = if ($UseImperial) { "mph" } else { "m/s" }

$weather_data | Select-Object @{Name="ID";Expression={$_.ID}},
    @{Name="Name_Display";Expression={$_.Name_Display}},
    @{Name="Dist($unit_dist)";Expression={$_.Distance}},
    @{Name="Elev($unit_elev)";Expression={$_.Elevation}},
    @{Name="Timestamp";Expression={$_.Timestamp}},
    @{Name="Temp($unit_temp)";Expression={$_.Temp}},
    @{Name="%RH";Expression={[int]$_.Humidity}},
    @{Name="Wind($unit_wind)";Expression={$_.WindSpeed}},
    @{Name="WindDir";Expression={$_.WindDir}},
    @{Name="Conditions";Expression={$_.Conditions}},
    @{Name="Pptn1Hr";Expression={$_.Precip_1hr}},
    @{Name="Pptn3Hr";Expression={$_.Precip_3hr}} |
Format-Table -AutoSize

# Check for hidden columns (all values are blank or same)
$possibly_hidden = @()

# Define shorthand map: column label → property name in object
$column_checks = @{
    'Temp(F)'    = 'Temp'
    '%RH'        = 'Humidity'
    'Wind(mph)'  = 'WindSpeed'
    'WindDir'    = 'WindDir'
    'Conditions' = 'Conditions'
    'Pptn1Hr'    = 'Precip_1hr'
    'Pptn3Hr'    = 'Precip_3hr'
}

foreach ($label in $column_checks.Keys) {
    $prop = $column_checks[$label]
    $values = $weather_data | ForEach-Object { $_.$prop }
    if (AllBlank $values -or AllSame $values) {
        $possibly_hidden += $label
    }
}

if ($possibly_hidden.Count -gt 0) {
    Write-Host "NOTE: Powershell tables may not show columns w/ all identical or all blank values: " -NoNewLine -ForegroundColor Yellow
    Write-Host ($possibly_hidden -join ", ") -ForegroundColor Green
 }

# Final export
$csv_path  = Join-Path $OutPath "weather_stations.csv"
$json_path = Join-Path $OutPath "weather_stations.json"

$weather_data | Select-Object ID, Name, Distance, Elevation, Timestamp, Temp, HumidityRaw, WindSpeed, WindDir, Conditions, Precip_1hr, Precip_3hr |
    Export-Csv -NoTypeInformation -Path $csv_path
$weather_data | Select-Object ID, Name, Distance, Elevation, Timestamp, Temp, HumidityRaw, WindSpeed, WindDir, Conditions, Precip_1hr, Precip_3hr |
    ConvertTo-Json -Depth 3 | Out-File -Encoding UTF8 $json_path
if ($possibly_hidden.Count -gt 0) {
    Write-Host "However, all data results written to:`n - $csv_path`n - $json_path"
} else {
    Write-Host "Data results written to:`n - $csv_path`n - $json_path"
}
